@@ -38,28 +38,29 @@ func TestMaxEncodedLen(t *testing.T) {
 	testSet := []struct {
 		in, out int64
 	}{
-		{in: 0, out: 1},
-		{in: 1 << 24, out: 1<<24 + int64(binary.PutVarint([]byte{binary.MaxVarintLen32: 0}, int64(1<<24))) + literalExtraSize(1<<24)},
-		{in: MaxBlockSize, out: math.MaxUint32},
-		{in: math.MaxUint32 - binary.MaxVarintLen32 - literalExtraSize(math.MaxUint32), out: math.MaxUint32},
-		{in: math.MaxUint32 - 9, out: -1},
-		{in: math.MaxUint32 - 8, out: -1},
-		{in: math.MaxUint32 - 7, out: -1},
-		{in: math.MaxUint32 - 6, out: -1},
-		{in: math.MaxUint32 - 5, out: -1},
-		{in: math.MaxUint32 - 4, out: -1},
-		{in: math.MaxUint32 - 3, out: -1},
-		{in: math.MaxUint32 - 2, out: -1},
-		{in: math.MaxUint32 - 1, out: -1},
-		{in: math.MaxUint32, out: -1},
-		{in: -1, out: -1},
-		{in: -2, out: -1},
+		0:  {in: 0, out: 1},
+		1:  {in: 1 << 24, out: 1<<24 + int64(binary.PutVarint([]byte{binary.MaxVarintLen32: 0}, int64(1<<24))) + literalExtraSize(1<<24)},
+		2:  {in: MaxBlockSize, out: math.MaxUint32},
+		3:  {in: math.MaxUint32 - binary.MaxVarintLen32 - literalExtraSize(math.MaxUint32), out: math.MaxUint32},
+		4:  {in: math.MaxUint32 - 9, out: -1},
+		5:  {in: math.MaxUint32 - 8, out: -1},
+		6:  {in: math.MaxUint32 - 7, out: -1},
+		7:  {in: math.MaxUint32 - 6, out: -1},
+		8:  {in: math.MaxUint32 - 5, out: -1},
+		9:  {in: math.MaxUint32 - 4, out: -1},
+		10: {in: math.MaxUint32 - 3, out: -1},
+		11: {in: math.MaxUint32 - 2, out: -1},
+		12: {in: math.MaxUint32 - 1, out: -1},
+		13: {in: math.MaxUint32, out: -1},
+		14: {in: -1, out: -1},
+		15: {in: -2, out: -1},
 	}
 	// 32 bit platforms have a different threshold.
 	if maxInt == math.MaxInt32 {
-		testSet[2].out = -1
+		testSet[2].out = math.MaxInt32
 		testSet[3].out = -1
 	}
+	t.Log("Maxblock:", MaxBlockSize, "reduction:", intReduction)
 	// Test all sizes up to maxBlockSize.
 	for i := int64(0); i < maxBlockSize; i++ {
 		testSet = append(testSet, struct{ in, out int64 }{in: i, out: i + int64(binary.PutVarint([]byte{binary.MaxVarintLen32: 0}, i)) + literalExtraSize(i)})
@@ -69,7 +70,7 @@ func TestMaxEncodedLen(t *testing.T) {
 		want := tt.out
 		got := int64(MaxEncodedLen(int(tt.in)))
 		if got != want {
-			t.Fatalf("input: %d, want: %d, got: %d", tt.in, want, got)
+			t.Errorf("test %d: input: %d, want: %d, got: %d", i, tt.in, want, got)
 		}
 	}
 }
@@ -1651,12 +1652,40 @@ func downloadBenchmarkFiles(b testing.TB, basename string) (errRet error) {
 	return nil
 }
 
+func TestEstimateBlockSize(t *testing.T) {
+	var input []byte
+	for i := 0; i < 100; i++ {
+		EstimateBlockSize(input)
+		input = append(input, 0)
+	}
+}
+
 func benchFile(b *testing.B, i int, decode bool) {
 	if err := downloadBenchmarkFiles(b, testFiles[i].filename); err != nil {
 		b.Fatalf("failed to download testdata: %s", err)
 	}
 	bDir := filepath.FromSlash(*benchdataDir)
 	data := readFile(b, filepath.Join(bDir, testFiles[i].filename))
+	if !decode {
+		b.Run("est-size", func(b *testing.B) {
+			if n := testFiles[i].sizeLimit; 0 < n && n < len(data) {
+				data = data[:n]
+			}
+			b.SetBytes(int64(len(data)))
+			b.ReportAllocs()
+			b.ResetTimer()
+			b.RunParallel(func(pb *testing.PB) {
+				for pb.Next() {
+					_ = EstimateBlockSize(data)
+				}
+			})
+			sz := float64(EstimateBlockSize(data))
+			if sz > 0 {
+				b.ReportMetric(100*sz/float64(len(data)), "pct")
+				b.ReportMetric(sz, "B")
+			}
+		})
+	}
 	b.Run("block", func(b *testing.B) {
 		if n := testFiles[i].sizeLimit; 0 < n && n < len(data) {
 			data = data[:n]
@@ -2172,3 +2201,51 @@ func TestMatchLen(t *testing.T) {
 		}
 	}
 }
+
+func BenchmarkDecodeBlockSingle(b *testing.B) {
+	for i := range testFiles {
+		b.Run(fmt.Sprint(i, "-", testFiles[i].label), func(b *testing.B) {
+			if err := downloadBenchmarkFiles(b, testFiles[i].filename); err != nil {
+				b.Fatalf("failed to download testdata: %s", err)
+			}
+			bDir := filepath.FromSlash(*benchdataDir)
+			data := readFile(b, filepath.Join(bDir, testFiles[i].filename))
+			if testFiles[i].sizeLimit > 0 && len(data) > testFiles[i].sizeLimit {
+				data = data[:testFiles[i].sizeLimit]
+			}
+			benchDecode(b, data)
+		})
+	}
+}
+
+func BenchmarkDecodeBlockParallel(b *testing.B) {
+	for i := range testFiles {
+		b.Run(fmt.Sprint(i, "-", testFiles[i].label), func(b *testing.B) {
+			benchFile(b, i, true)
+		})
+	}
+}
+
+func BenchmarkEncodeBlockSingle(b *testing.B) {
+	for i := range testFiles {
+		b.Run(fmt.Sprint(i, "-", testFiles[i].label), func(b *testing.B) {
+			if err := downloadBenchmarkFiles(b, testFiles[i].filename); err != nil {
+				b.Fatalf("failed to download testdata: %s", err)
+			}
+			bDir := filepath.FromSlash(*benchdataDir)
+			data := readFile(b, filepath.Join(bDir, testFiles[i].filename))
+			if testFiles[i].sizeLimit > 0 && len(data) > testFiles[i].sizeLimit {
+				data = data[:testFiles[i].sizeLimit]
+			}
+			benchEncode(b, data)
+		})
+	}
+}
+
+func BenchmarkEncodeBlockParallel(b *testing.B) {
+	for i := range testFiles {
+		b.Run(fmt.Sprint(i, "-", testFiles[i].label), func(b *testing.B) {
+			benchFile(b, i, false)
+		})
+	}
+}
